﻿using System;
using System.Security.Cryptography;

namespace com.bodurov.SortEvolver.Processor
{
    public class CryptoRnd : Random
    {
        
        private readonly RNGCryptoServiceProvider _rng;

        public CryptoRnd()
        {
            this._rng = new RNGCryptoServiceProvider();
        }

        private int GetInt32FromRNG()
        {
            var inegerBytes = new byte[4];
            this.NextBytes(inegerBytes);
            int i = BitConverter.ToInt32(inegerBytes, 0);
            if (i < 0)
            {
                i += 0x7fffffff;
            }
            return i;
        }

        /// <summary>
        /// Fills the elements of a specified array of bytes with random numbers. 
        /// </summary>
        /// <param name="buffer">An array of bytes to contain random numbers.</param>
        public override void NextBytes(byte[] buffer)
        {
            this._rng.GetBytes(buffer);
        }



        /// <summary>
        /// Returns a random number between 0.0 and 1.0. 
        /// </summary>
        /// <returns>A double-precision floating point number greater than or equal to 0.0, and less than 1.0.</returns>
        protected override double Sample()
        {
            return (this.GetInt32FromRNG() * 4.6566128752457969E-10);
        }

        private double GetSampleForLargeRange()
        {
            int num = this.GetInt32FromRNG();
            if ((((this.GetInt32FromRNG() % 2) == 0) ? 1 : 0) != 0)
            {
                num = -num;
            }
            double num2 = num;
            num2 += 2147483646.0;
            return (num2 / 4294967293);
        }

        /// <summary>
        /// Returns a random number within a specified range.
        /// </summary>
        /// <param name="minValue">The INCLUSIVE lower bound of the random number returned. </param>
        /// <param name="maxValue">The EXCLUSIVE upper bound of the random number returned. maxValue must be greater than or equal to minValue.</param>
        /// <returns>A 32-bit signed integer greater than or equal to minValue and less than maxValue; that is, the range of return values includes minValue but not maxValue. If minValue equals maxValue, minValue is returned.</returns>
        public override int Next(int minValue, int maxValue)
        {
            if (minValue > maxValue)
            {
                throw new ArgumentOutOfRangeException("minValue", "minValue cannot be higher than maxValue");
            }
            long num = maxValue - (long)minValue;
            if (num <= 0x7fffffffL)
            {
                return (((int)(this.Sample() * num)) + minValue);
            }
            double d = this.GetSampleForLargeRange();
            return (((int)((long)(d * num))) + minValue);
        }

        /// <summary>
        /// Tests probability for example cr.TestChangce(0.1) will return true in 10% of cases
        /// </summary>
        public bool TestChance(double d)
        {
            if (d < 0 || d > 1) throw new ArgumentException("The value passed to CryptoRnd TestChance must be between 0 and 1");
            if (d == 0) return false;
            if (d == 1) return true;
            return this.Sample() <= d;
        }
    }
}


